;********************************************************************************
;*    lib_mot_moway_10.inc                                                      *
;*    version: 1.0                                                              *
;*    Esta librera contiene las funciones necesarias para gestionar el mdulo  *
;*    motor del robot Moway con programas en ensamblado                         *
;*    Copyright (C) 2007  Bizintek Innova S.L.                                  *
;********************************************************************************
;*    This program is free software; you can redistribute it and/or modify      *
;*    it under the terms of the GNU General Public License as published by      *
;*    the Free Software Foundation; either version 2 of the License, or         *
;*    (at your option) any later version.                                       *
;*                                                                              *
;*    This program is distributed in the hope that it will be useful,           *
;*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
;*    GNU General Public License for more details.                              *
;*                                                                              *
;*    You should have received a copy of the GNU General Public License along   *
;*    with this program; if not, write to the Free Software Foundation, Inc.,   *
;*    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.               *
;********************************************************************************

;Elimina los warning de cambio de banco
 ERRORLEVEL -302 

;********************
;*	DEFINICIONES	*
;********************

;*****************************
;*Configuracion del patillaje*
;*****************************

	;PORTC
#define		SDA				PORTC,1
#define		SCL				PORTC,0 

	;PORTB
#define		MOT_END_TRIS	TRISB,5

	;TRISC
#define		SCL_TRIS		TRISC,0
#define		SDA_TRIS		TRISC,1	

	;TRISB
#define		MOT_END			PORTB,5

;*****************
;*Variables fijas*
;*****************

;Direccin del motor
#define	  MOT_DIR		  0x10




;Comandos
;Byte MOT_COMAND                nbit     7   6   5   4   3   2   1   0
;                                       --------------------------------
;Comandos movimientos
#define   COM_STR         0x00   ;       "0   0   0   0   0   0   CT  RL"
#define   COM_CHA_VEL     0x01   ;       "0   0   0   0   0   1   CT  RL"
#define   COM_ROT         0x02   ;       "0   0   0   0   1   0   CT  RL"
#define   COM_CUR         0x03   ;       "0   0   0   0   1   1   CT  RL"
#define   COM_STOP        0x04   ;       "0   0   0   1   0   0   CT  RL"

;Comandos informacin
#define   STATUS_T        0x00   ;       "1   0   0   0   0   0   0   0 "
#define   STATUS_A        0x01   ;       "1   0   0   0   0   0   0   1 "
#define   STATUS_V_R      0x02   ;       "1   0   0   0   0   0   1   0 "
#define   STATUS_V_L      0x03   ;       "1   0   0   0   0   0   1   1 "
#define   STATUS_D_R      0x04   ;       "1   0   0   0   0   1   0   0 "
#define   STATUS_D_L      0x05   ;       "1   0   0   0   0   1   0   1 "
#define   STATUS_KM       0x06   ;       "1   0   0   0   0   1   1   0 "

;Comandos reset
#define   RST_T           0x00   ;       "1   1   1   1   0   0   0   0 "
#define   RST_D           0x01   ;       "1   1   1   1   0   0   0   1 "
#define   RST_KM          0x02   ;       "1   1   1   1   0   0   1   0 "







;* Variable MOT_STATUS			 		;Estado del comando
#define   COMOK				0	 		;Comando correcto (1) o incorrecto (0)
#define   DWRONG			1    		;Datos incorrectos (1) o correctos (0)		

;  Variable MOT_CON 			 		;Registro de control de motores
#define	  FWDBACK			0	 		;Comando hacia adelante (1) o hacia atras (0)		
#define	  RL				1	 		;Comando derecha (1) o izquierda (0)
#define	  COMTYPE		    2	 		;Comando de tiempo (1) o, de distancia o angulo (0)

;* Variable FLAGS_MOT					;Registro interno
#define	  DIR				0			
#define	  COM				1 	
#define	  DAT1				2
#define	  DAT2				3
#define	  DAT3				4
#define	  NO_ACK_INTENTO	5


;****************
;*	VARIABLES	*
;****************

;* Variables usuario
MOT_STATUS			   	EQU	0x54		;Estado del comando
MOT_VEL					EQU 0x53		;Velocidad deseada
MOT_T_DIST_ANG			EQU 0x52		;Tiempo-Distancia-Angulo dependiendo de COMTYPE y la funcin 
MOT_CENWHEEL			EQU	0x51		;Rotar sobre el centro o una de las ruedas
MOT_RAD					EQU	0x50		;Radio de curvatura
MOT_CON 			    EQU	0x4F		;Registro de control de motores
MOT_RST_COM				EQU	0x4E		;Comando reset
MOT_STATUS_COM		   	EQU	0x4D		;Comando status
MOT_STATUS_DATA_1		EQU	0x4C		;Segundo dato leido por el comando MOT_STAT (Valido cuando se lee el cuentakilometros, sino 0xFF)
MOT_STATUS_DATA_0		EQU	0x4B		;Primer dato leido por el comando MOT_STAT

;Variable MOT internas
MOT_COMAND				EQU	0x4A
FLAGS_MOT				EQU	0x49
INTENTOS				EQU	0x48
INTER_MOT				EQU	0x47
MOT_DATA_0				EQU	0x46
MOT_DATA_1				EQU 0x45
MOT_DATA_2				EQU 0x44	
MOT_DATA_3				EQU 0x43

;* Variables de las funciones de bajo nivel
I_BYTE					EQU 0x42		
O_BYTE					EQU 0x41		
_N						EQU	0x40		



;************
;*	CDIGO	*
;************


;*****************************************************
;*                   MOT_CONFIG                      *
;*****************************************************
;*Descripcin:Esta funcin se encarga de configurar  *
;*el PIC para que puede comunicarse con el mdulo    *
;*motor                                              *
;*****************************************************
;*Variables de entrada:                              *
;*Variables de salida:                               *
;*****************************************************

MOT_CONFIG
	call	MOT_INIT
	banksel	TRISC
	bsf		SDA_TRIS
	bsf		SCL_TRIS
	bsf		MOT_END_TRIS
	bcf		STATUS,RP1
	bcf		STATUS,RP0
;;Inicializacin de variables
	clrf	MOT_VEL					
	clrf	MOT_T_DIST_ANG			
	clrf	MOT_CENWHEEL		
	clrf	MOT_RAD				
	clrf	MOT_CON 			  
	clrf	MOT_RST_COM			
	clrf	MOT_STATUS_COM		   	
	clrf	MOT_STATUS_DATA_1		
	clrf 	MOT_STATUS_DATA_0		
	clrf 	MOT_COMAND			
	clrf 	FLAGS_MOT				
	clrf 	INTENTOS				
	clrf 	INTER_MOT				
	clrf 	MOT_DATA_0				
	clrf 	MOT_DATA_1				
	clrf 	MOT_DATA_2					
	clrf 	MOT_DATA_3				
	clrf 	I_BYTE							
	clrf 	O_BYTE							
	clrf 	_N			
	goto	MOT_FIN
;*****************************************************	






;*****************************************************
;*       				 MOT_STR   		              *
;*****************************************************
;*Descripcin: Comando para ir recto. Hay que        *
;*especificar velocidad, sentido, tipo de comando y  *
;*el tiempo o la distancia a recorrer                *
;*****************************************************
;*Variables de entrada:- MOT_VEL   : 0  - 100        *
;*					   - MOT_CON:					 *
;*                     	 - FWDBACK : Adelante   (1)  * 
;*									 Atras      (0)  *
;*                       - COMTYPE : Tiempo	    (1)  *
;*									 Distancia  (0)  *   
;*                     - MOT_T_DIST_ANG: 0  - 255	 *
;*Variables de salida:                               *
;*Salida:              - MOT_STATUS:		         *
;*                     	 - COMOK   : Enviado    (1)	 *
;*									 No enviado (0)	 *
;*                     	 - DWRONG  : Datos no OK(1)  *
;*								     Datos OK   (0)  *
;*****************************************************

MOT_STR
	call	MOT_INIT

    ;Comprobacin de datos correctos

    ;Velocidad debe estar entre 0 y 100
	movfw	MOT_VEL
	addlw   .155							
 	addlw   .101
	btfsc	STATUS,C
	goto	$+3
	bsf		MOT_STATUS,DWRONG
	goto	MOT_FIN

 	;Preparacin de datos para ser mandados por I2C

    ;El comando a mandar es COM_STR (0x00)
	movlw	COM_STR
	movwf	MOT_COMAND

	;Se mandan COMTYPE en el mismo byte de MOT_COMAND
	clrc
	btfsc	MOT_CON,COMTYPE	
	bsf		STATUS,C
	rlf		MOT_COMAND,F

	;Se mandan RL en el mismo byte de MOT_COMAND
	clrc	
	rlf		MOT_COMAND,F
	movfw	MOT_COMAND
	movwf	MOT_DATA_0

	;En LSB de velocidad se empaquetara el sentido
	clrc
	btfsc	MOT_CON,FWDBACK	
	bsf		STATUS,C
	rlf		MOT_VEL,W
	movwf	MOT_DATA_1	

	movfw	MOT_T_DIST_ANG
	movwf	MOT_DATA_2

	;Se mandan los datos (MOT_DATA) por I2C que 
	;anteriormente han sido actualizados
	call	MOT_SND_I2C_3D
	btfss	W,0
	call	STOP		

	goto	MOT_FIN
;*****************************************************






;*****************************************************
;*       		     MOT_CHA_VEL  	                 *
;*****************************************************
;*Descripcin: Comando para cambiar la velocidad de  *
;*uno de los motores. Para ello hay que especificar  *
;*velocidad, sentido, motor izquierdo o derecho,tipo *
;*de comando y el tiempo o la distancia a recorrer.  *
;*****************************************************
;*Variables de entrada:- MOT_VEL   : 0  - 100        *
;*					   - MOT_CON:					 *
;*                     	 - FWDBACK : Adelante   (1)  * 
;*									 Atras      (0)  *
;*						 - RL	   : Derecha	(1)	 *
;*								   : Izquierda  (0)  *
;*                       - COMTYPE : Tiempo	    (1)  *
;*									 Distancia  (0)  *   
;*                     - MOT_T_DIST_ANG: 0  - 255	 *
;*Variables de salida:                               *
;*Salida:              - MOT_STATUS:	             *
;*                     	 - COMOK   : Enviado    (1)	 *
;*									 No enviado (0)	 *
;*                     	 - DWRONG  : Datos no OK(1)  *
;*								     Datos OK   (0)  *
;*****************************************************

MOT_CHA_VEL
	call	MOT_INIT

    ;Comprobacin de datos correctos

    ;Velocidad debe estar entre 0 y 100
	movfw	MOT_VEL
	addlw   .155							
 	addlw   .101
	btfsc	STATUS,C
	goto	$+3
	bsf		MOT_STATUS,DWRONG
	goto	MOT_FIN

 	;Preparacin de datos para ser mandados por I2C

    ;El comando a mandar es COM_CHA_VEL (0x01)
	movlw	COM_CHA_VEL
	movwf	MOT_COMAND

	;Se mandan COMTYPE en el mismo byte de MOT_COMAND
	clrc
	btfsc	MOT_CON,COMTYPE	
	bsf		STATUS,C
	rlf		MOT_COMAND,F

	;Se mandan RL en el mismo byte de MOT_COMAND
	clrc	
	btfsc	MOT_CON,RL	
	bsf		STATUS,C
	rlf		MOT_COMAND,F
	movfw	MOT_COMAND
	movwf	MOT_DATA_0

	;En LSB de velocidad se empaquetara el sentido
	clrc
	btfsc	MOT_CON,FWDBACK	
	bsf		STATUS,C
	rlf		MOT_VEL,W
	movwf	MOT_DATA_1

	movfw	MOT_T_DIST_ANG
	movwf	MOT_DATA_2

	;Se mandan los datos (MOT_DATA) por I2C que 
	;anteriormente han sido actualizados
	call	MOT_SND_I2C_3D
	btfss	W,0
	call	STOP		

	goto	MOT_FIN
;*****************************************************







;*****************************************************
;*       		       MOT_ROT   	                 *
;*****************************************************
;*Descripcin: Comando para rotar. Para ello hay que *
;*especificar velocidad, sentido, rotacin           *
;*respecto al centro o a la rueda,tipo de comando y  *
;*el tiempo o el angulo a recorrer                   *
;*****************************************************
;*Variables de entrada:- MOT_VEL   : 0  - 100        *
;*					   - MOT_CON:					 *
;*                     	 - FWDBACK : Adelante   (1)  * 
;*									 Atras      (0)  *
;*						 - RL	   : Derecha	(1)	 *
;*								   : Izquierda  (0)  *
;*                       - COMTYPE : Tiempo	    (1)  *
;*									 Distancia  (0)  *
;*					   - MOT_CENWHEEL: Centro  (0x01)*
;*									   Rueda   (0x00)*
;*                     - MOT_T_DIST_ANG: 0  - 255	 *
;*Variables de salida:                               *
;*Salida:              - MOT_STATUS:	             *
;*                     	 - COMOK   : Enviado    (1)	 *
;*									 No enviado (0)	 *
;*                     	 - DWRONG  : Datos no OK(1)  *
;*								     Datos OK   (0)  *
;*****************************************************

MOT_ROT
	call	MOT_INIT
	
    ;Comprobacin de datos correctos

    ;Velocidad debe estar entre 0 y 100
	movfw	MOT_VEL
	addlw   .155							
 	addlw   .101
	btfsc	STATUS,C
	goto	$+3
	bsf		MOT_STATUS,DWRONG
	goto	MOT_FIN

	;El angulo debe estar entre 0 y 100
	btfsc	MOT_CON,COMTYPE
	goto	$+8
	movfw	MOT_T_DIST_ANG
	addlw   .155							
 	addlw   .101
	btfsc	STATUS,C
	goto	$+3
	bsf		MOT_STATUS,DWRONG
	goto	MOT_FIN

	;Preparacin de datos para ser mandados por I2C

    ;El comando a mandar es COM_CHA_VEL (0x02)
	movlw	COM_ROT
	movwf	MOT_COMAND

	;Se mandan COMTYPE en el mismo byte de MOT_COMAND
	clrc
	btfsc	MOT_CON,COMTYPE	
	bsf		STATUS,C
	rlf		MOT_COMAND,F

	;Se mandan RL en el mismo byte de MOT_COMAND
	clrc	
	btfsc	MOT_CON,RL	
	bsf		STATUS,C
	rlf		MOT_COMAND,F


	movfw	MOT_COMAND
	movwf	MOT_DATA_0

	;En LSB de velocidad se empaquetara el sentido
	clrc
	btfsc	MOT_CON,FWDBACK	
	bsf		STATUS,C
	rlf		MOT_VEL,W
	movwf	MOT_DATA_1

	movfw	MOT_CENWHEEL
	movwf	MOT_DATA_2
	movfw	MOT_T_DIST_ANG
	movwf	MOT_DATA_3

	;Se mandan los datos (MOT_DATA) por I2C que 
	;anteriormente han sido actualizados
	call	MOT_SND_I2C_4D
	btfss	W,0
	call	STOP							

	goto	MOT_FIN
;*****************************************************






;*****************************************************
;*       		       MOT_CUR  	                 *
;*****************************************************
;*Descripcin: Comando para dar curvas. Para ello hay*
;*que especificar velocidad, sentido, radio,         *
;*izquierda o derecha, tipo de comando y el tiempo o *
;*la distancia a recorrer.                           *
;*****************************************************
;*Variables de entrada:- MOT_VEL   : 0  - 100        *
;*					   - MOT_CON:					 *
;*                     	 - FWDBACK : Adelante   (1)  * 
;*									 Atras      (0)  *
;*						 - RL	   : Derecha	(1)	 *
;*								   : Izquierda  (0)  *
;*                       - COMTYPE : Tiempo	    (1)  *
;*									 Distancia  (0)  *  
;*					   - MOT_RAD   : 0  - 100  		 *
;*                     - MOT_T_DIST_ANG: 0  - 255	 *
;*Variables de salida:                               *
;*Salida:              - MOT_STATUS:	             *
;*                     	 - COMOK   : Enviado    (1)	 *
;*									 No enviado (0)	 *
;*                     	 - DWRONG  : Datos no OK(1)  *
;*								     Datos OK   (0)  *
;*****************************************************
MOT_CUR
	call	MOT_INIT

    ;Comprobacin de datos correctos

    ;Velocidad debe estar entre 0 y 100
	movfw	MOT_VEL
	addlw   .155							
 	addlw   .101
	btfsc	STATUS,C
	goto	$+3
	bsf		MOT_STATUS,DWRONG
	goto	MOT_FIN

	;Comprobacin si el radio es correcto
	movfw 	MOT_RAD
	subwf	MOT_VEL,W
	addlw   .155							
 	addlw   .101
	btfsc	STATUS,C
	goto	$+3
	bsf		MOT_STATUS,DWRONG
	goto	MOT_FIN
	movfw 	MOT_RAD
	addwf	MOT_VEL,W
	addlw   .155							
 	addlw   .101
	btfsc	STATUS,C
	goto	$+3
	bsf		MOT_STATUS,DWRONG
	goto	MOT_FIN
	bcf		MOT_STATUS,DWRONG
	

	;Preparacin de datos para ser mandados por I2C

    ;El comando a mandar es COM_CHA_VEL (0x03)	
	movlw	COM_CUR
	movwf	MOT_COMAND

	;Se mandan COMTYPE en el mismo byte de MOT_COMAND
	clrc
	btfsc	MOT_CON,COMTYPE	
	bsf		STATUS,C
	rlf		MOT_COMAND,F

	;Se mandan RL en el mismo byte de MOT_COMAND
	clrc	
	btfsc	MOT_CON,RL	
	bsf		STATUS,C
	rlf		MOT_COMAND,F

	movfw	MOT_COMAND
	movwf	MOT_DATA_0

	;En LSB de velocidad se empaquetara el sentido
	clrc
	btfsc	MOT_CON,FWDBACK	
	bsf		STATUS,C
	rlf		MOT_VEL,W
	movwf	MOT_DATA_1

	movfw	MOT_RAD
	movwf	MOT_DATA_2

	movfw	MOT_T_DIST_ANG
	movwf	MOT_DATA_3

	;Se mandan los datos (MOT_DATA) por I2C que 
	;anteriormente han sido actualizados
	call	MOT_SND_I2C_4D
	btfss	W,0
	call	STOP		

	goto	MOT_FIN
;*****************************************************






;*****************************************************
;*                     MOT_STOP                      *
;*****************************************************
;*Descripcin: Comando para parar.                   *
;*****************************************************
;*Variables de entrada:                              *
;*Variables de salida:                               *
;*Salida:              - MOT_STATUS:	             *
;*                     	 - COMOK   : Enviado    (1)	 *
;*									 No enviado (0)	 *
;*****************************************************
MOT_STOP
	call	MOT_INIT

	;Se prepara el comando STOP
	movlw	COM_STOP
	movwf	MOT_COMAND

	;No se manda COMTYPE ni RL
	clrc	
	rlf		MOT_COMAND,F
	clrc	
	rlf		MOT_COMAND,F

	movfw	MOT_COMAND
	movwf	MOT_DATA_0

	;Se mandan los datos (MOT_DATA) por I2C que 
	;anteriormente han sido actualizados
	call	MOT_SND_I2C_1D
	btfss	W,0
	call	STOP		

	goto	MOT_FIN
;*****************************************************





;*****************************************************
;*                       MOT_RST                     *
;*****************************************************
;*Descripcin: Comando para resetear tiempo,         *
;*distancia o el cuentakilometros.                   *
;*****************************************************
;*Variables de entrada:- MOT_RST_COM:                *
;*                         -RST_T                    *
;*                         -RST_D                    *
;*                         -RST_KM                   *
;*Variables de salida:                               *
;*Salida:              - MOT_STATUS:	             *
;*                     	 - COMOK   : Enviado    (1)	 *
;*									 No enviado (0)	 *
;*****************************************************
MOT_RST
	call	MOT_INIT

	;Se prepara el comando reset
	movfw	MOT_RST_COM
	iorlw	0xF0
	movwf	MOT_RST_COM
	movfw	MOT_RST_COM
	movwf	MOT_DATA_0
	
	;Se mandan los datos (MOT_DATA) por I2C que 
	;anteriormente han sido actualizados
	call	MOT_SND_I2C_1D
	btfss	W,0
	call	STOP		

	goto	MOT_FIN
;*****************************************************





;*****************************************************
;*                     MOT_FDBCK	                 *
;*****************************************************
;*Descripcin: Comando para conocer varios parmetros*
;*del mdulo motor: tiempo, angulo recorrido,        *
;*velocidad de los dos motores, distancia recorrida  *
;*por los dos motores y valor del cuentakilometros   *
;*(2 variables de salida).                           *
;*****************************************************
;*Variables de entrada:- MOT_STATUS_COM:             *
;*                         -STATUS_T                 *
;*                         -STATUS_A                 *
;*                         -STATUS_V_R               *
;*                         -STATUS_V_L               *
;*                         -STATUS_D_R               *
;*                         -STATUS_D_L               *
;*                         -STATUS_KM                *
;*Variables de salida: - MOT_STATUS_DATA_0           *
;*                     - MOT_STATUS_DATA_1           *
;*Salida:              - MOT_STATUS:		         *
;*                     	 - COMOK   : Enviado    (1)	 *
;*									 No enviado (0)	 *
;*****************************************************
MOT_FDBCK
	call	MOT_INIT

	bsf		MOT_STATUS_COM,7 
	movfw	MOT_STATUS_COM
	movwf	MOT_DATA_0


	;Se mandan los datos (MOT_DATA) por I2C que 
	;anteriormente han sido actualizados
	call	MOT_SND_I2C_1D
	call	MOT_SND_I2C_1D
	btfsc	W,0
	goto	MOT_FIN
	call	STOP			


	;Lectura de los datos
	call	I2CSTART		;Comenzar envo i2c
SND_I2C_NO_ACK_READ
	movlw	0x11
	call	MANDAR_I2C
	btfsc	FLAGS_MOT,NO_ACK_INTENTO
	goto	SND_I2C_NO_ACK_READ
	btfss	MOT_STATUS,COMOK
	goto	MOT_TIME_OUT_1D
	bsf		FLAGS_MOT,DIR

	call 	IN_BYTE			;Recibe el primer byte
	call	OUT_ACK			;Manda ACK
	movfw	I_BYTE
	movwf	MOT_STATUS_DATA_0

	call 	IN_BYTE			;Recibe el primer byte
	call	OUT_NACK		;Manda NACK: Terminacin de envio
	movfw	I_BYTE
	movwf	MOT_STATUS_DATA_1

	call	STOP	

	goto	MOT_FIN	
;*****************************************************









;*****************************************************
;*              	 MOT_SND_I2C_1D                  *
;*****************************************************
;*Descripcin:Funcin de comunicacin por I2C de bajo*
;*nivel encargado de mandar un solo dato.            *
;*****************************************************
;*Variables de entrada:- MOT_DATA_0                  *
;*Variables de salida:                               *
;*Salida:              - 0: Envo correcto           *
;*                     - 1: Envio incorrecto         *
;*****************************************************
MOT_SND_I2C_1D

SND_I2C_NO_ACK_1D

	call	I2CSTART		;Comenzar envo i2c

	movlw	MOT_DIR
	call	MANDAR_I2C
	btfsc	FLAGS_MOT,NO_ACK_INTENTO
	goto	SND_I2C_NO_ACK_1D
	btfss	MOT_STATUS,COMOK
	goto	MOT_TIME_OUT_1D

	
	movfw	MOT_DATA_0
	call	MANDAR_I2C
	btfsc	FLAGS_MOT,NO_ACK_INTENTO
	goto	SND_I2C_NO_ACK_1D
	btfss	MOT_STATUS,COMOK
	goto	MOT_TIME_OUT_1D

	
	retlw	.0
MOT_TIME_OUT_1D
	
	retlw	.1
;*****************************************************





;*****************************************************
;*       			MOT_SND_I2C_3D               	 *
;*****************************************************
;*Descripcin:Funcin de comunicacin por I2C de bajo*
;*nivel encargado de mandar tres datos.              *
;*****************************************************
;*Variables de entrada:- MOT_DATA_0-3                *
;*Variables de salida:                               *
;*Salida:              - 0: Envo correcto           *
;*                     - 1: Envio incorrecto         *
;*****************************************************
MOT_SND_I2C_3D

SND_I2C_NO_ACK_3D

	call	I2CSTART	;Comenzar envo i2c

	movlw	MOT_DIR
	call	MANDAR_I2C
	btfsc	FLAGS_MOT,NO_ACK_INTENTO
	goto	SND_I2C_NO_ACK_3D
	btfss	MOT_STATUS,COMOK
	goto	MOT_TIME_OUT_3D

	
	movfw	MOT_DATA_0
	call	MANDAR_I2C
	btfsc	FLAGS_MOT,NO_ACK_INTENTO
	goto	SND_I2C_NO_ACK_3D
	btfss	MOT_STATUS,COMOK
	goto	MOT_TIME_OUT_3D

	
	movfw	MOT_DATA_1
	call	MANDAR_I2C
	btfsc	FLAGS_MOT,NO_ACK_INTENTO
	goto	SND_I2C_NO_ACK_3D
	btfss	MOT_STATUS,COMOK
	goto	MOT_TIME_OUT_3D

	
	movfw	MOT_DATA_2
	call	MANDAR_I2C
	btfsc	FLAGS_MOT,NO_ACK_INTENTO
	goto	SND_I2C_NO_ACK_3D
	btfss	MOT_STATUS,COMOK
	goto	MOT_TIME_OUT_3D


	retlw	.0
MOT_TIME_OUT_3D
	
	retlw	.1
;*****************************************************





;*****************************************************
;*     				MOT_SND_I2C_4D      			 *
;*****************************************************
;*Descripcin:Funcin de comunicacin por I2C de bajo*
;*nivel encargado de mandar cuatro datos.            *
;*****************************************************
;*Variables de entrada:- MOT_DATA_0-4                *
;*Variables de salida:                               *
;*Salida:              - 0: Envo correcto           *
;*                     - 1: Envio incorrecto         *
;*****************************************************
MOT_SND_I2C_4D

SND_I2C_NO_ACK_4D

	call	I2CSTART	;Comenzar envo i2c

	movlw	MOT_DIR
	call	MANDAR_I2C
	btfsc	FLAGS_MOT,NO_ACK_INTENTO
	goto	SND_I2C_NO_ACK_4D
	btfss	MOT_STATUS,COMOK
	goto	MOT_TIME_OUT_4D

	
	movfw	MOT_DATA_0
	call	MANDAR_I2C
	btfsc	FLAGS_MOT,NO_ACK_INTENTO
	goto	SND_I2C_NO_ACK_4D
	btfss	MOT_STATUS,COMOK
	goto	MOT_TIME_OUT_4D

	
	movfw	MOT_DATA_1
	call	MANDAR_I2C
	btfsc	FLAGS_MOT,NO_ACK_INTENTO
	goto	SND_I2C_NO_ACK_4D
	btfss	MOT_STATUS,COMOK
	goto	MOT_TIME_OUT_4D

	
	movfw	MOT_DATA_2
	call	MANDAR_I2C
	btfsc	FLAGS_MOT,NO_ACK_INTENTO
	goto	SND_I2C_NO_ACK_4D
	btfss	MOT_STATUS,COMOK
	goto	MOT_TIME_OUT_4D


	movfw	MOT_DATA_3
	call	MANDAR_I2C
	btfsc	FLAGS_MOT,NO_ACK_INTENTO
	goto	SND_I2C_NO_ACK_4D
	btfss	MOT_STATUS,COMOK
	goto	MOT_TIME_OUT_4D


	retlw	.0

MOT_TIME_OUT_4D
	
	retlw	.1
;*****************************************************







;*********************************
;*********************************
;**	INSTRUCCIONES DE BAJO NIVEL	**
;*********************************
;*********************************

;************************************************
;*MOT_INIT	:Inicializa funciones				*
;************************************************
MOT_INIT
	
	bcf		STATUS,RP1
	bcf		STATUS,RP0
	clrf	FLAGS_MOT
	call	MOT_INTER_CALL
	clrf	MOT_STATUS
	movlw	.10
	movwf	INTENTOS
	return

;************************************************
;* MOT_FIN	:Finaliza las funciones				*
;************************************************
MOT_FIN
	call	MOT_INTER_RETURN
	return

;************************************************
;*MOT_INTER_CALL:Checkea si las interrupciones 	*
;*			     estan habilitadas. Si lo estan	*
;*				 las inhabilita					*
;************************************************

MOT_INTER_CALL

	bcf		STATUS,RP1
	bcf		STATUS,RP0
	movfw	INTCON
	movwf	INTER_MOT
	bcf		INTCON,GIE
	return
	

;*************************************************
;*MOT_INTER_RETURN:Checkea si las interrupciones *
;*			       estan habilitadas. Si lo estan*
;*				 las inhabilita					 *
;*************************************************

MOT_INTER_RETURN
	bcf		STATUS,RP1
	bcf		STATUS,RP0	
	movfw	INTER_MOT
	btfsc	INTER_MOT,7	
	goto	MOT_GIE_UNO
	goto	MOT_GIE_CERO
MOT_GIE_UNO
	bsf		INTCON,GIE
	return
MOT_GIE_CERO
	bcf		INTCON,GIE
	return
	


;************************************************
;* MANDAR_I2C		:Manda lo que esta en el 	*
;*					 acumulador por I2C			*
;************************************************
;*	W				->	Acumulador				*
;************************************************
;*	FLAGS_MOT			->	Flag de control		*
;*	FLAGS_MOT(NO_ENVIADO)->	Comando no enviado 	*
;*	FLAGS_MOT(NO_ACK_INTENTO)->	No ACK			*
;************************************************

MANDAR_I2C
	movwf	O_BYTE
	call 	OUT_BYTE	;Enviar 
	call	IN_ACK		;Comprueba ACK
	xorlw	0xFF
	btfss	STATUS,Z
	goto 	NO_ACK
	bsf		MOT_STATUS,COMOK
	bcf		FLAGS_MOT,NO_ACK_INTENTO
	return

NO_ACK
	call 	STOP
	decfsz	INTENTOS
	goto	NO_ACK_INTENTOS
	bcf		FLAGS_MOT,NO_ACK_INTENTO
	bcf		MOT_STATUS,COMOK
	return

NO_ACK_INTENTOS
	bsf		FLAGS_MOT,NO_ACK_INTENTO
	return	


;************************************************
;* IN_ACK			:Lee el ACK o el NACK del 	*
;*				     SLAVE. No hace falta 		*
;*					 configurar SDA como entrada*
;*					 (esta rutina lo hace 		*
;*					 automaticamente)   		*
;************************************************
;*	ACK				->	Devuelve 0xFF			*
;*	NACK			->	Devuelve 0x00			*
;************************************************
IN_ACK			

	banksel	TRISC
	bsf		SDA_TRIS
	bcf		STATUS,RP1
	bcf		STATUS,RP0	
	
	banksel	TRISC
	bsf 	SCL_TRIS		
	bcf		STATUS,RP1
	bcf		STATUS,RP0	

	banksel	PORTC
	btfss 	SDA	
	goto 	IN_ACK_RET
IN_NACK_RET
	banksel	TRISC
	bcf 	SDA_TRIS		

	banksel	TRISC
	bcf 	SCL_TRIS		
	bcf		STATUS,RP1
	bcf		STATUS,RP0	

	bcf		STATUS,RP1
	bcf		STATUS,RP0	
	retlw 	0x00
IN_ACK_RET
	banksel	TRISC
	bcf 	SDA_TRIS		

	banksel	TRISC
	bcf 	SCL_TRIS		
	bcf		STATUS,RP1
	bcf		STATUS,RP0	

	bcf		STATUS,RP1
	bcf		STATUS,RP0	
	retlw 	0xFF

;************************************************
;* OUT_BYTE:		:Manda byte creando a su vez* 
;*					 el reloj en SCL 			*
;************************************************
;*	O_BYTE			->	Byte a mandar			*
;************************************************

OUT_BYTE:
			
	movlw 	.8
	movwf 	_N
OUT_BIT:
	bcf 	STATUS,C		
	rlf 	O_BYTE, F		
	btfss 	STATUS, C		
	goto 	OUT_ZERO
	goto 	OUT_ONE
OUT_ZERO:

	banksel	TRISC
	bcf 	SDA_TRIS		;Hacemos SDA como salida. 
	bcf		STATUS,RP1		;Nivel lgico 0.
	bcf		STATUS,RP0	
		
	banksel	TRISC
	bsf 	SCL_TRIS		;Hacemos SCL como entrada. 	
	bcf		STATUS,RP1		;Nivel lgico 1.
	bcf		STATUS,RP0
	
	banksel	TRISC
	bcf 	SCL_TRIS		;Hacemos SCL como salida. 
	bcf		STATUS,RP1		;Nivel lgico 0.
	bcf		STATUS,RP0	
	
	goto 	OUT_CONT
OUT_ONE:

	banksel	TRISC
	bsf 	SDA_TRIS		;Hacemos SDA como entrada. 
	bcf		STATUS,RP1		;Nivel lgico 1.
	bcf		STATUS,RP0	
		
	banksel	TRISC
	bsf 	SCL_TRIS		;Hacemos SCL como entrada. 
	bcf		STATUS,RP1		;Nivel lgico 1.
	bcf		STATUS,RP0
	
	banksel	TRISC
	bcf 	SCL_TRIS		;Hacemos SCL como salida. 
	bcf		STATUS,RP1		;Nivel lgico 0.
	bcf		STATUS,RP0	

	goto 	OUT_CONT
OUT_CONT:
	decfsz 	_N, F			; decrementa el ndice
	goto 	OUT_BIT
	retlw 	0	

;************************************************
;* IN_BYTE:		:	 Recibe byte creando a su vez* 
;*					 el reloj en SCL 			*
;************************************************
;*	I_BYTE			->	Byte a recibir			*
;************************************************

IN_BYTE						
	clrf 	I_BYTE
	movlw 	.8
	movwf 	_N				; pone el ndice a 8	
	call 	HIGH_SDA		
IN_BIT
	call 	HIGH_SCL
	btfss	SCL				; Espera hasta que el slave deje el clock libre 
	goto	$-1				; Hasta que el reloj no este "stretch"
	btfss 	SDA				; comprueba SDA
	goto 	IN_ZERO
	goto 	IN_ONE
IN_ZERO
	bcf 	STATUS, C		; borra el carry existente
	rlf 	I_BYTE, F		; i_byte = (i_byte << 1) | 0
	goto 	CONT_IN
IN_ONE
	bcf 	STATUS, C		; borra el carry existente
	rlf 	I_BYTE, F		
	incf 	I_BYTE, F		; i_byte = (i_byte << 1) | 1
	goto 	CONT_IN
CONT_IN
	call 	LOW_SCL		
	decfsz 	_N, F
	goto 	IN_BIT
	call 	LOW_SDA	
	retlw 	0

;************************************************
;* OUT_ACK:		:	 Saca una seal de ACK		*	 
;************************************************

OUT_ACK
	call 	LOW_SDA
	call 	CLOCK_PULSE
	call 	HIGH_SDA
	retlw 	0

;************************************************
;* OUT_NACK:		:	 Saca una seal de ACK		*	 
;************************************************

OUT_NACK
	call 	HIGH_SDA
	call 	CLOCK_PULSE
	call 	HIGH_SDA
	retlw 	0

;************************************************
;* I2CSTART			:Condicion de START para 	*
;*					 empezar la comunicacion por*
;*	      			 I2C						*
;************************************************
;*												*
;*												*
;************************************************

I2CSTART:	
	banksel	PORTC
	movfw	PORTC
	andlw	b'11111100'
	movwf	PORTC

	call 	HIGH_SDA
	call 	HIGH_SCL
	call 	LOW_SDA			;Hacemos 0 SDA 
	call 	LOW_SCL			;mientras SLC esta en alto.
	return

;************************************************
;* STOP				:Condicion de STOP para 	*
;*					 terminar la comunicacion 	*
;*					 por I2C					*
;************************************************
;*												*
;*												*
;************************************************
STOP:
	banksel	TRISC
	bsf 	SCL_TRIS		;Hacemos SCL como entrada. 
	bcf		STATUS,RP1		;Nivel lgico 1.
	bcf		STATUS,RP0	

	banksel	TRISC
	bsf 	SDA_TRIS		;Hacemos SDA como entrada. 
	bcf		STATUS,RP1		;Nivel lgico 1.
	bcf		STATUS,RP0	
	
	bcf		STATUS,RP1
	bcf		STATUS,RP0	
	return



;************************************************
;* Rutinas de bajo nivel						*
;*					 							*
;*					 							*
;************************************************
;*												*
;*												*
;************************************************


CLOCK_PULSE:			
	banksel	TRISC
	bsf 	SCL_TRIS		;Hacemos SCL como entrada.
	bcf		STATUS,RP1		;Nivel lgico 1.
	bcf		STATUS,RP0	

	banksel	TRISC
	bcf 	SCL_TRIS		;Hacemos SDL como salida. 
	bcf		STATUS,RP1		;Nivel lgico 0.
	bcf		STATUS,RP0	
	retlw 	0

HIGH_SDA:				
	banksel	TRISC
	bsf 	SDA_TRIS		;Hacemos SDA como entrada. 
	bcf		STATUS,RP1		;Nivel lgico 1.
	bcf		STATUS,RP0	
	retlw 	0

LOW_SDA:		
	banksel	TRISC
	bcf 	SDA_TRIS		;Hacemos SDA como salida. 
	bcf		STATUS,RP1		;Nivel lgico 0.
	bcf		STATUS,RP0	
	retlw 	0

HIGH_SCL:
	banksel	TRISC
	bsf 	SCL_TRIS		;Hacemos SCL como entrada. 
	bcf		STATUS,RP1		;Nivel lgico 1.
	bcf		STATUS,RP0	
	retlw 	0

LOW_SCL:	
	banksel	TRISC
	bcf 	SCL_TRIS		;Hacemos SCL como salida. 
	bcf		STATUS,RP1		;Nivel lgico 0.
	bcf		STATUS,RP0	
	retlw 	0




