;********************************************************************************
;*    lib_sen_moway_10.inc                                                      *
;*    version: 1.0                                                              *
;*    Esta librera contiene las funciones necesarias para gestionar los        *
;*    sensores del robot Moway en ensamblador                                   *
;*    Copyright (C) 2007  Bizintek Innova S.L.                                  *
;*                                                                              *
;********************************************************************************
;*    This program is free software; you can redistribute it and/or modify      *
;*    it under the terms of the GNU General Public License as published by      *
;*    the Free Software Foundation; either version 2 of the License, or         *
;*    (at your option) any later version.                                       *
;*                                                                              *
;*    This program is distributed in the hope that it will be useful,           *
;*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
;*    GNU General Public License for more details.                              *
;*                                                                              *
;*    You should have received a copy of the GNU General Public License along   *
;*    with this program; if not, write to the Free Software Foundation, Inc.,   *
;*    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.               *
;********************************************************************************

;Elimina los warning de cambio de banco
 ERRORLEVEL -302 

;********************
;*   DEFINICIONES   *
;********************

; PORTA
#define   LIGHT_SEN      	   PORTA,0
#define   IR_RX_R         	   PORTA,1
#define   LINE_RX_R            PORTA,2
#define   IR_RX_L              PORTA,3
#define   LED_TOP_RED          PORTA,4
#define   LINE_RX_L            PORTA,5

; PORTB
#define   LINE_TX              PORTB,1
#define   IR_TX                PORTB,2
#define   LED_R                PORTB,4
#define   LED_TOP_GREEN        PORTB,6

; PORTC
#define   LED_L                PORTC,6


; TRISA
#define   LIGHT_SEN_TRIS       TRISA,0
#define   IR_RX_R_TRIS         TRISA,1
#define   LINE_RX_R_TRIS       TRISA,2
#define   IR_RX_L_TRIS         TRISA,3
#define   LED_TOP_RED_TRIS     TRISA,4
#define   LINE_RX_L_TRIS       TRISA,5

; TRISB
#define   LINE_TX_TRIS         TRISB,1
#define   IR_TX_TRIS           TRISB,2
#define   LED_R_TRIS           TRISB,4
#define   LED_TOP_GREEN_TRIS   TRISB,6

; TRISC
#define   LED_L_TRIS           TRISC,6



;* Variable FLAGS_MOT
#define		SENOK				1  		;Lectura del sensor obstaculos correcta (1) o incorrecta (0)


;****************
;*	VARIABLES	*
;****************

;Variables usuario
SEN_LIGHT_P 			EQU 0x3B		;Porcentage de luz incidente en sensor de luz			
SEN_LINE_L  			EQU	0x3A		;Salida del sensor de lnea izquierdo
SEN_LINE_R  			EQU	0x39		;Salida del sensor de lnea derecho
SEN_OBS_L				EQU 0x38		;Salida del sensor de obstculos izquierdo
SEN_OBS_R				EQU 0x37		;Salida del sensor de obstculos derecho
SEN_STATUS				EQU 0x3C		;Estado de lectura de sensor

;Variables SEN internas	
INTER_SEN				EQU	0x35
DELAY_1					EQU 0x3E
DELAY_1_1				EQU 0x3D




;************
;*	CDIGO	*
;************

;*****************************************************
;*                   SEN_CONFIG                      *
;*****************************************************
;*Descripcin: La funcin configura las E/S del PIC y*
;*a su vez configura el ADC							 *
;*****************************************************
;*Variables de entrada:                              *
;*Variables de salida:                               *
;*****************************************************

SEN_CONFIG

	call	SEN_INTER_CALL

	;Configuracin PORT_A
	banksel	TRISA
	bsf		LIGHT_SEN_TRIS       
	bsf		IR_RX_R_TRIS         
   	bsf		LINE_RX_R_TRIS       
   	bsf		IR_RX_L_TRIS         
   	bcf		LED_TOP_GREEN_TRIS  
   	bsf		LINE_RX_L_TRIS       
	;Configuracin PORT_B
	bcf		LINE_TX_TRIS         
  	bcf		IR_TX_TRIS           
    bcf		LED_R_TRIS           
    bcf		LED_TOP_RED_TRIS    
	;Configuracin PORT_C
	bcf		LED_L_TRIS         

	;Configuracin del ADC
	banksel	ADCON1
	movlw	b'01000110'
	movwf	ADCON1

    ;No necesita resistencias de pull-up en el
    ;PORTB
	banksel	OPTION_REG
	bsf		OPTION_REG,NOT_RBPU
	
	bcf		STATUS,RP1
	bcf		STATUS,RP0

	;Por defecto el LED verde est apagado
	bsf		LED_TOP_GREEN	
	;Por defecto el LED rojo est apagado
	bsf		LED_TOP_RED	
	;Por defecto el LED verde est apagado
	bcf		LED_L	
	;Por defecto el LED rojo est apagado
	bcf		LED_R	
	

	;Por defecto el LED infrarrojo est apagado
	bcf		IR_TX

	;Borrado de variables
	clrf	SEN_LIGHT_P
	clrf	SEN_OBS_L
	clrf	SEN_OBS_R
	clrf	SEN_LINE_L
	clrf	SEN_LINE_R

	clrf	INTER_SEN
	clrf	DELAY_1
	clrf	DELAY_1_1

	call	SEN_INTER_RETURN

	return
;*****************************************************






	
;*****************************************************
;*                  SEN_LIGHT                        *
;*****************************************************
;*Descripcin:Devuelve el porcentage de luz          *
;*recibida por el sensor de luz (0-100)              *
;*****************************************************
;*Variables de entrada:                              *
;*Variables de salida: -SEN_LIGHT_P                  *
;*****************************************************

SEN_LIGHT
	call	SEN_INTER_CALL

   	;Se selecciona el canal0 del ADC y se espera el
   	;tiempo de adquisicin del convertidor analgico
   	;digital
	banksel	ADCON1
	movlw	b'01000010'
	movwf	ADCON1

	movlw	b'01000001'		
	banksel	ADCON0
	movwf	ADCON0
	call	Delay_100us

	;Lectura y calculo del porcentage de luz incidente
	bsf		ADCON0,GO
	btfsc	ADCON0,GO	
	goto	$-1
	movfw	ADRESH			
	movwf	SEN_LIGHT_P
	bcf		STATUS,C
	rrf		SEN_LIGHT_P,F	
	bcf		ADCON0,ADON
	banksel	ADCON1
	movlw	b'01000110'
	movwf	ADCON1

	call	SEN_INTER_RETURN
	return
;*****************************************************







;*****************************************************
;*                 SEN_OBS_DIG                       *
;*****************************************************
;*Descripcin: Devuelve si se ha detectado un        *
;*obstculo en la izquierda o derecha del robot      *
;*****************************************************
;*Variables de entrada:                              *
;*Variables de salida: -SEN_OBS_R                    *
;*                     -SEN_OBS_L                    *
;*Salida:			   -SEN_STATUS					 *
;*						 -SENOK: 					 *
;*		 	              -0:Detectado seal externa *
;*                        (Moway o interferecia)     *
;*                        -1:Lectura correcta        *
;*****************************************************
SEN_OBS_DIG	
	call	SEN_INTER_CALL

	;Comprobacin de la no existencia de seal externa
    ;indeseada
	btfsc	IR_RX_R
	goto	COLISION_DIG
	
	btfsc	IR_RX_L
	goto	COLISION_DIG	

	;Se manda pulso de luz infrarroja para
   	;la detecin de obstculo
	bsf		IR_TX
	call	Delay_65us

  	;Comprobacin si el pulso a rebotado en
   	;algn objeto
	clrf	SEN_OBS_R
	movlw	.1
	btfsc	IR_RX_R
	movwf	SEN_OBS_R
	clrf	SEN_OBS_L
	btfsc	IR_RX_L
	movwf	SEN_OBS_L

	;Finalizacin del pulso de luz infrarroja
	bcf		IR_TX
	call	Delay_30us

	;Comprobacin si se ha leido el pulso mandado
   	;o interferencia externa
	btfsc	IR_RX_R
	goto	COLISION_DIG
	
	btfsc	IR_RX_L
	goto	COLISION_DIG	
		
	bsf		SEN_STATUS,SENOK

	call	Delay_1500us

	call	SEN_INTER_RETURN
	
	return

COLISION_DIG
	call	Delay_1500us
	bcf		SEN_STATUS,SENOK
	clrf	SEN_OBS_R
	clrf	SEN_OBS_L
	call	SEN_INTER_RETURN
	return
;*****************************************************






;*****************************************************
;*                 SEN_OBS_ANALOG                    *
;*****************************************************
;*Descripcin: Devuelve si se ha detectado un        *
;*obstculo en la izquierda o derecha del robot en   *
;*valor proporcional a la distancia.                 *
;*****************************************************
;*Variables de entrada:                              *
;*Variables de salida: -SEN_OBS_R                    *
;*                     -SEN_OBS_L                    *
;*Salida:			   -SEN_STATUS					 *
;*						 -SENOK: 					 *
;*		 	              -0:Detectado seal externa *
;*                        (Moway o interferecia)     *
;*                        -1:Lectura correcta        *
;*****************************************************

SEN_OBS_ANALOG	
	call	SEN_INTER_CALL

	;Comprobacin de la no existencia de seal externa
    ;indeseada en la derecha
	btfsc	IR_RX_R
	goto	COLISION_ANALOG_1
	
	btfsc	IR_RX_L
	goto	COLISION_ANALOG_1	

 	;Se configura los pines del PORTA como analgicos
    ;y se elige el canal 1. Se espera el tiempo de
    ;adquisicin que en este caso es corto por la
    ;resistencia de salida del sensor IR
	banksel	ADCON1
	movlw	b'01000010'
	movwf	ADCON1
	movlw	b'01001001'		
	banksel	ADCON0
	movwf	ADCON0

	;Se manda pulso de luz infrarroja para
    ;la detecin de obstculo en la derecha
	bsf		IR_TX
	call	Delay_65us

 	;Lectura del valor del sensor derecho
	bsf		ADCON0,GO
	btfsc	ADCON0,GO	
	goto	$-1
	movfw	ADRESH		
	movwf	SEN_OBS_R

	;Finalizacin del pulso de luz infrarroja
	bcf		IR_TX
	call	Delay_30us
	banksel	ADCON1
	movlw	b'01000110'
	movwf	ADCON1
	bcf		STATUS,RP1
	bcf		STATUS,RP0

    ;Comprobacin si se ha leido el pulso mandado
    ;o interferencia externa
	btfsc	IR_RX_R
	goto	COLISION_ANALOG_2
	
	btfsc	IR_RX_L
	goto	COLISION_ANALOG_2

	call	Delay_1500us

	
	;Comprobacin de la no existencia de seal externa
    ;indeseada en la izquierda
	btfsc	IR_RX_R
	goto	COLISION_ANALOG_1
	
	btfsc	IR_RX_L
	goto	COLISION_ANALOG_1


	;Se configura los pines del PORTA como analgicos
   	;y se elige el canal 1. Se espera el tiempo de
   	;adquisicin que en este caso es corto por la
   	;resistencia de salida del sensor IR
	banksel	ADCON1
	movlw	b'01000010'
	movwf	ADCON1
	movlw	b'01011001'		
	banksel	ADCON0
	movwf	ADCON0

	;Se manda pulso de luz infrarroja para
   	;la detecin de obstculo en la izquierda
	bsf		IR_TX
	call	Delay_65us

	;Lectura del valor del sensor izquierdo
	bsf		ADCON0,GO
	btfsc	ADCON0,GO	
	goto	$-1
	movfw	ADRESH		
	movwf	SEN_OBS_L

	;Finalizacin del pulso de luz infrarroja
	bcf		IR_TX
	call	Delay_30us
	banksel	ADCON1
	movlw	b'01000110'
	movwf	ADCON1
	bcf		STATUS,RP1
	bcf		STATUS,RP0

	;Comprobacin si se ha leido el pulso mandado
   	;o interferencia externa
	btfsc	IR_RX_R
	goto	COLISION_ANALOG_2
	
	btfsc	IR_RX_L
	goto	COLISION_ANALOG_2


	call	Delay_1500us

	bsf		SEN_STATUS,SENOK

	call	SEN_INTER_RETURN

	return	

COLISION_ANALOG_1
	bcf		SEN_STATUS,SENOK
	clrf	SEN_OBS_L
	clrf	SEN_OBS_R
	call	SEN_INTER_RETURN
	return

COLISION_ANALOG_2
	call	Delay_1500us
	bcf		SEN_STATUS,SENOK
	clrf	SEN_OBS_L
	clrf	SEN_OBS_R
	call	SEN_INTER_RETURN
	return
;*****************************************************







;*****************************************************
;*               SEN_LINE_DIG                       *
;*****************************************************
;*Descripcin: Devuelve si se ha detectado una       *
;*lnea negra sobre suelo claro                      *
;*****************************************************
;*Variables de entrada:                              *
;*Variables de salida: -SEN_LINE_R	                 *
;*                     -SEN_LINE_L	                 *
;*****************************************************

SEN_LINE_DIG
	call	SEN_INTER_CALL

 	;Se manda pulso de luz infrarroja para
    ;la detecin de lnea
	bsf		LINE_TX
	call 	Delay_900us
	
	;Lectura de la luz infrarroja rebotada en el suelo
	clrf	SEN_LINE_R
	movlw	.1		
	btfsc	LINE_RX_R
	movwf	SEN_LINE_R
	clrf	SEN_LINE_L	
	btfsc	LINE_RX_L
	movwf	SEN_LINE_L

	;Finalizacin del pulso de luz infrarroja
	bcf		LINE_TX

	call	SEN_INTER_RETURN
	return	
;*****************************************************







;*****************************************************
;*               SEN_LINE_ANALOG                     *
;*****************************************************
;*Descripcin: Devuelve el valor de la claridad del  *
;*suelo. Se puede distinguir colores dependiendo del *
;*material.                                          *
;*****************************************************
;*Variables de entrada:                              *
;*Variables de salida: -SEN_LINE_R         	         *
;*                     -SEN_LINE_L                   *
;*****************************************************

SEN_LINE_ANALOG
	call	SEN_INTER_CALL

	;Configuramos las entradas como analgicas
	banksel	ADCON1
	movlw	b'01000010'
	movwf	ADCON1

	;Se manda pulso de luz infrarroja para
    ;la detecin de lnea
	movlw	b'01010001'		
	banksel	ADCON0
	movwf	ADCON0
	bsf		LINE_TX
	call 	Delay_900us

	;Lectura del sensor derecho
	bsf		ADCON0,GO
	btfsc	ADCON0,GO	
	goto	$-1
	movfw	ADRESH		
	movwf	SEN_LINE_R
	
	;Cambio de canal del ADC	
	movlw	b'01100001'		
	banksel	ADCON0
	movwf	ADCON0
	call	Delay_100us

	;Lectura del sensor izquierdo
	bsf		ADCON0,GO
	btfsc	ADCON0,GO	
	goto	$-1
	movfw	ADRESH		
	movwf	SEN_LINE_L	
	
	;Finalizacin del pulso de luz infrarroja
	bcf		LINE_TX
	banksel	ADCON1
	movlw	b'01000110'
	movwf	ADCON1

	call	SEN_INTER_RETURN
	return	
;*****************************************************







;*****************************************************
;*                   LED_xx_xx                       *
;*****************************************************
;*Descripcin: Funciones de manejo de LED del robot  *
;*****************************************************
;*Variables de entrada:                              *
;*Variables de salida:                               *
;*****************************************************
;Encendido de led delatero derecho
LED_R_ON	
	call	SEN_INTER_CALL
	bsf		LED_R
	call	SEN_INTER_RETURN
	return	
;Apagado de led delatero derecho
LED_R_OFF	
	call	SEN_INTER_CALL
	bcf		LED_R
	call	SEN_INTER_RETURN
	return	

;Parpadeo de led delantero derecho
LED_R_ON_OFF
	call	SEN_INTER_CALL
	bcf		STATUS,RP1
	bcf		STATUS,RP0
	bsf		LED_R
	call	Delay_50ms
	bcf		LED_R
	call	Delay_50ms
	call	SEN_INTER_RETURN
	return	
;Encendido de led delantero izquierdo
LED_L_ON	
	call	SEN_INTER_CALL
	bsf		LED_L
	call	SEN_INTER_RETURN
	return	

;Apagado de led derantero izquierdo
LED_L_OFF
	call	SEN_INTER_CALL
	bcf		LED_L
	call	SEN_INTER_RETURN
	return	
;Parpadeo de led delantero izquierdo
LED_L_ON_OFF
	call	SEN_INTER_CALL
	bcf		STATUS,RP1
	bcf		STATUS,RP0
	bsf		LED_L
	call	Delay_50ms
	bcf		LED_L
	call	Delay_50ms
	call	SEN_INTER_RETURN
	return	

;Encendido de led superior rojo
LED_TOP_RED_ON	
	call	SEN_INTER_CALL
	bcf		LED_TOP_RED
	call	SEN_INTER_RETURN
	return	

;Apagado de led superior rojo
LED_TOP_RED_OFF
	call	SEN_INTER_CALL
	bsf		LED_TOP_RED
	call	SEN_INTER_RETURN
	return	

;Parpadeo de led superior rojo
LED_TOP_RED_ON_OFF
	call	SEN_INTER_CALL
	bcf		STATUS,RP1
	bcf		STATUS,RP0
	bcf		LED_TOP_RED
	call	Delay_50ms
	bsf		LED_TOP_RED
	call	Delay_50ms
	call	SEN_INTER_RETURN
	return	

;Encendido de led superior verde
LED_TOP_GREEN_ON	
	call	SEN_INTER_CALL
	bcf		LED_TOP_GREEN
	call	SEN_INTER_RETURN
	return	

;Apagado de led superior verde
LED_TOP_GREEN_OFF
	call	SEN_INTER_CALL
	bsf		LED_TOP_GREEN
	call	SEN_INTER_RETURN
	return	

;Parpadeo de led superior verde
LED_TOP_GREEN_ON_OFF
	call	SEN_INTER_CALL
	bcf		STATUS,RP1
	bcf		STATUS,RP0
	bcf		LED_TOP_GREEN
	call	Delay_50ms
	bsf		LED_TOP_GREEN
	call	Delay_50ms
	call	SEN_INTER_RETURN
	return	
;*****************************************************





;*****************************
;*****************************
;**	FUNCIONES AUXILIARES	**
;*****************************
;*****************************




;************************************************
;*SEN_INTER_CALL:Checkea si las interrupciones 	*
;*			     estan habilitadas. Si lo estan	*
;*				 las inhabilita					*
;************************************************

SEN_INTER_CALL
	bcf		STATUS,RP1
	bcf		STATUS,RP0
	movfw	INTCON
	movwf	INTER_SEN
	bcf		INTCON,GIE
	return
;************************************************

	



;*************************************************
;*SEN_INTER_RETURN:Checkea si las interrupciones *
;*			       estan habilitadas. Si lo estan*
;*				   las inhabilita				 *
;*************************************************

SEN_INTER_RETURN
	bcf		STATUS,RP1
	bcf		STATUS,RP0	
	movfw	INTER_SEN
	btfsc	INTER_SEN,7	
	goto	SEN_GIE_UNO
	goto	SEN_GIE_CERO

SEN_GIE_UNO
	bsf		INTCON,GIE
	return
SEN_GIE_CERO
	bcf		INTCON,GIE
	return
;************************************************




;*************************************************
;*Delay_xxxx:Temporizaciones necesarias para la  * 
;*			 correcta captura de datos			 *
;*************************************************

Delay_50ms
	movlw	0x0E
	movwf	DELAY_1
	movlw	0x28
	movwf	DELAY_1_1
Delay_50ms_0
	decfsz	DELAY_1, f
	goto	$+2
	decfsz	DELAY_1_1, f
	goto	Delay_50ms_0
	goto	$+1
	nop
	return
;************************************************
Delay_100us
	movlw	0x1F
	movwf	DELAY_1
Delay_100us_0
	decfsz	DELAY_1, f
	goto	Delay_100us_0
	goto	$+1
	return
;************************************************
Delay_900us
	movlw	0xB2
	movwf	DELAY_1
	movlw	0x01
	movwf	DELAY_1_1
Delay_900us_0
	decfsz	DELAY_1, f
	goto	$+2
	decfsz	DELAY_1_1, f
	goto	Delay_900us_0
	goto	$+1
	nop
	return
;************************************************
Delay_65us
	movlw	0x14
	movwf	DELAY_1
Delay_65us_0
	decfsz	DELAY_1, f
	goto	Delay_65us_0
	return
;************************************************
Delay_30us
	movlw	0x08
	movwf	DELAY_1
Delay_30us_0
	decfsz	DELAY_1, f
	goto	Delay_30us_0
	nop
	return
;************************************************
Delay_1500us
	movlw	0x2A
	movwf	DELAY_1
	movlw	0x02
	movwf	DELAY_1_1
Delay_1500us_0
	decfsz	DELAY_1, f
	goto	$+2
	decfsz	DELAY_1_1, f
	goto	Delay_1500us_0
	goto	$+1
	nop
	return
;************************************************



